<?php

namespace SM\Service;

use SM\Factory\AgendaServicosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use SM\Util\Utilities;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class AgendaServicosService
{
    protected $entityManager;
    protected $agendaServicosFactory;
    protected $Environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, AgendaServicosFactory $agendaServicosFactory, Environment $Environment, NewCode $newCode)
    {
        $this->agendaServicosFactory = $agendaServicosFactory;
        $this->connection             = $entityManager->getConnection();
        $this->Environment            = $Environment;
        $this->newCode                = $newCode;
    }

    public function getAgendaServicos($params)
    {
        $cdcliente =  '';
        $cdclienteC =  '';
        if (isset($params["CDCLIENTE"])) {
            $cdcliente = "AND CDCLIENTE =  '" . $params["CDCLIENTE"] . "'";
            $cdclienteC = "AND C.CDCLIENTE =  '" . $params["CDCLIENTE"] . "'";
        }

        $nrproposfas =  '';
        $nrproposfasC = '';
        if (isset($params["NRPROPOSFAS"])) {
            $nrproposfas = "AND NRPROPOSFAS =  '" . $params["NRPROPOSFAS"] . "'";
        }

        $nrverpropfas = '';
        $nrverpropfasC = '';
        if (isset($params["NRVERPROPFAS"])) {
            $nrverpropfas = "AND NRVERPROPFAS =  '" . $params["NRVERPROPFAS"] . "'";
            $nrverpropfasC = "AND C.NRVERPROPFAS =  '" . $params["NRVERPROPFAS"] . "'";
        }

        $nrcontratofct = '';
        if (isset($params["NRSEQCONTRATO"])) {
            $nrcontratofct = "AND CT.NRSEQCONTRATO =  '" . $params["NRSEQCONTRATO"] . "'";
        }

        $dtAgendamento = '';
        if (isset($params['USR_DTAGENDZERASALDO'])) {
            $dataInicial = $params['USR_DTAGENDZERASALDO'][0];
            $dataFinal = array_key_exists("1", $params['USR_DTAGENDZERASALDO']) ? $params['USR_DTAGENDZERASALDO'][1] : null;

            if ($dataFinal != null) {
                $dtAgendamento = $dtAgendamento . " AND C.USR_DTAGENDZERASALDO BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
            } else {
                $data = $dataInicial;
                $dtAgendamento = $dtAgendamento . " AND C.USR_DTAGENDZERASALDO BETWEEN '" . $data . "' AND TO_DATE('" . $data . "')+30";
            }
        }
        return $this->agendaServicosFactory->getAgendaServicos($cdcliente, $nrproposfas, $nrverpropfas, $cdclienteC, $nrproposfasC, $nrverpropfasC, $nrcontratofct, $dtAgendamento);
    }

    public function getCliente()
    {
        return $this->agendaServicosFactory->getCliente();
    }

    public function getPropostaAgendaServicos($params)
    {
        $dataParams = array(
            "P_CDCLIENTE" => $params["CDCLIENTE"],
        );
        return $this->agendaServicosFactory->getPropostaAgendaServicos($dataParams);
    }

    public function zerarSaldoDeHoras($params)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        try {
            $nomeoperador = $this->agendaServicosFactory->getNomeOperador(array(
                "CDOPERADOR" => $cdoperador,
            ));
            $dataAtual = date('d-m-Y');
            $dataAtualAux = strtotime($dataAtual);
            $dataInput = $params["USR_DTAGENDZERASALDO"];
            $dataInputAux = strtotime($dataInput);
            if ($dataAtualAux == $dataInputAux) {
                if ($params["row"]["USR_QTSERVCONT_BKP"] == null) {
                    $bkpContrato = array(
                        "USR_QTSERVCONT_BKP"   => $params["row"]["QTSERVPEDC"],
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORG"                => $nrorg,
                        "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                        "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                        "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                        "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                    );
                    $this->agendaServicosFactory->updatebkpContratoServ($bkpContrato);
                }
                $saldoFinal = $params["row"]["QTSERVPEDC"] - $params["row"]["SALDO"];
                $dataParamsContrato = array(
                    "USR_DTAGENDZERASALDO" => $dataInput,
                    "QTSERVCONT"           => $saldoFinal,
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORG"                => $nrorg,
                    "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                    "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                    "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                    "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                );
                $this->agendaServicosFactory->updateSaldoContratoServ($dataParamsContrato);
                $dataParamsLogContratoServ = array(
                    "NMTABELA"      => "CONTRATOSERV",
                    "NMRESPOPER"    => $nomeoperador["data"][0]["NMOPERADOR"],
                    "CDORESPOPER"   => $cdoperador,
                    "IDTIPOOPER"    => "1",
                    "DSNMUSUOPER"   => "sm-tek",
                    "DSMETUSUOPER"  => "zerarSaldoDeHoras",
                    "DSOPERACAO"    => $params["row"]["DSOPERACAO"],
                    "CDPESQITEM"    => $params["row"]["NRPROPOSFAS"] . $params["row"]["NRVERPROPFAS"] . $params["row"]["CDTIPOSERV"] . $params["row"]["NRSERVCONT"],
                );
                $this->agendaServicosFactory->insertLogOpera($dataParamsLogContratoServ);
            } elseif ($dataInputAux < $dataAtualAux) {
                throw new \Exception("ERRO");
            } else {
                $dataParamsContrato = array(
                    "USR_DTAGENDZERASALDO" => $dataInput,
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORG"                => $nrorg,
                    "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                    "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                    "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                    "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                );
                $this->agendaServicosFactory->updateDataContratoServ($dataParamsContrato);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function zerarSaldoDeHorasAction($params)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        
        try {
            $nomeoperador = $this->agendaServicosFactory->getNomeOperador(array(
                "CDOPERADOR" => $cdoperador,
            ));

            $dataAtual = date('d-m-Y');
            $dataAtualAux = strtotime($dataAtual);
            $dataInput = $params["USR_DTAGENDZERASALDO"];
            $dataInputAux = strtotime($dataInput);

            foreach ($params["row"] as $row) {
                if ($dataInputAux == $dataAtualAux) {
                    if ($row["USR_QTSERVCONT_BKP"] == null) {
                        $bkpContrato = array(
                            "USR_QTSERVCONT_BKP"   => $row["QTSERVPEDC"],
                            "CDOPERULTATU"         => $cdoperador,
                            "NRORG"                => $nrorg,
                            "CDTIPOSERV"           => substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1),
                            "CDCLIENTE"            => $row["CDCLIENTE"],
                            "NRPROPOSFAS"          => $row["NRPROPOSFAS"],
                            "NRVERPROPFAS"         => $row["NRVERPROPFAS"],
                        );
                        $this->agendaServicosFactory->updatebkpContratoServ($bkpContrato);
                    }
                    $saldo = $this->convertMinutes($row["SALDO"]);
                    $contratadas = $this->convertMinutes($row["QTSERVPEDC"]);
                    $saldoFinal = $contratadas - $saldo;
                    $dataParamsContrato = array(
                        "USR_DTAGENDZERASALDO" => $dataInput,
                        "QTSERVCONT"           => $saldoFinal,
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORG"                => $nrorg,
                        "CDTIPOSERV"           => substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1),
                        "CDCLIENTE"            => $row["CDCLIENTE"],
                        "NRPROPOSFAS"          => $row["NRPROPOSFAS"],
                        "NRVERPROPFAS"         => $row["NRVERPROPFAS"],
                    );
                    $this->agendaServicosFactory->updateSaldoContratoServ($dataParamsContrato);
                    $dsoperacao = "SALDO DE HORAS ALTERADO - Anterior: " . $row["QTSERVPEDC"] . " / Novo: 00:00";
                    $dataParamsLogContratoServ = array(
                        "NMTABELA"      => "CONTRATOSERV",
                        "NMRESPOPER"    => $nomeoperador["data"][0]["NMOPERADOR"],
                        "CDORESPOPER"   => $cdoperador,
                        "IDTIPOOPER"    => "1",
                        "DSNMUSUOPER"   => "sm-tek",
                        "DSMETUSUOPER"  => "zerarSaldoDeHoras",
                        "DSOPERACAO"    => $dsoperacao,
                        "CDPESQITEM"    => $row["NRPROPOSFAS"] . $row["NRVERPROPFAS"] . substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1) . $row["NRSERVCONT"],
                    );
                    $this->agendaServicosFactory->insertLogOpera($dataParamsLogContratoServ);
                } elseif ($dataInputAux < $dataAtualAux) {
                    throw new \Exception("ERRO");
                } else {
                    $dataParamsContrato = array(
                        "USR_DTAGENDZERASALDO" => $dataInput,
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORG"                => $nrorg,
                        "CDTIPOSERV"           => substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1),
                        "CDCLIENTE"            => $row["CDCLIENTE"],
                        "NRPROPOSFAS"          => $row["NRPROPOSFAS"],
                        "NRVERPROPFAS"         => $row["NRVERPROPFAS"],
                    );
                    $this->agendaServicosFactory->updateDataContratoServ($dataParamsContrato);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function zerarSaldoHoraAgendada($nrorg)
    {
        $dataParams = [
            "NRORG" => $nrorg
        ];
        $dataAgendada = $this->agendaServicosFactory->getPropostasAgendadas($dataParams);
        $ind = 0;
        foreach ($dataAgendada as $data) {
            $dataParams = [
                "row" => $data,
                "USR_DTAGENDZERASALDO"  => $data[$ind]["USR_DTAGENDZERASALDO"]
            ];
            $this->zerarSaldoDeHoras($dataParams);
            $ind++;
        }
    }

    public function convertMinutes($hora)
    {
        $f0 = explode(":", $hora);
        $minutes = ($f0[0] * 60) + $f0[1];
        $horas = $minutes / 60;
        return $horas;
    }

    public function editarSaldoDeHoras($params)
    {
        // debugApi($params);
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        try {
            if ($params["row"]["USR_QTSERVCONT_BKP"] == null) {
                $bkpContrato = array(
                    "USR_QTSERVCONT_BKP"   => $params["row"]["QTSERVPEDC"],
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORG"                => $nrorg,
                    "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                    "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                    "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                    "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                );
                $this->agendaServicosFactory->updatebkpContratoServ($bkpContrato);
            }
            $dataParamsContrato = array(
                "USR_DTAGENDZERASALDO" => $params["row"]["USR_DTAGENDZERASALDO"],
                "QTSERVCONT"           => $params["row"]["QTSERVPEDC"],
                "CDOPERULTATU"         => $cdoperador,
                "NRORG"                => $nrorg,
                "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
            );
            $this->agendaServicosFactory->updateSaldoContratoServ($dataParamsContrato);

            $nomeoperador = $this->agendaServicosFactory->getNomeOperador(array(
                "CDOPERADOR" => $cdoperador,
            ));

            $dataParamsLogContratoServ = array(
                "NMTABELA"      => "CONTRATOSERV",
                "NMRESPOPER"    => $nomeoperador["data"][0]["NMOPERADOR"],
                "CDORESPOPER"   => $cdoperador,
                "IDTIPOOPER"    => "1",
                "DSNMUSUOPER"   => "sm-tek",
                "DSMETUSUOPER"  => "editarSaldoDeHoras",
                "DSOPERACAO"    => $params["row"]["DSOPERACAO"],
                "CDPESQITEM"    => $params["row"]["NRPROPOSFAS"] . $params["row"]["NRVERPROPFAS"] . substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1) . $params["row"]["NRSERVCONT"],
            );
            $this->agendaServicosFactory->insertLogOpera($dataParamsLogContratoServ);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function reverterSaldoHoras($params)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        try {
            if ($params["row"]["USR_QTSERVCONT_BKP"] == null) {
                throw new \Exception("ERRO");
            } else {
                $bkpContrato = array(
                    "QTSERVCONT"           => $params["row"]["USR_QTSERVCONT_BKP"],
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORG"                => $nrorg,
                    "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                    "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                    "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                    "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                );
                $this->agendaServicosFactory->reverterSaldoHorasBkp($bkpContrato);
                $dataParamsContrato = array(
                    "USR_QTSERVCONT_BKP"   => null,
                    "USR_DTAGENDZERASALDO" => null,
                    "CDOPERULTATU"         => $cdoperador,
                    "NRORG"                => $nrorg,
                    "CDTIPOSERV"           => substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1),
                    "CDCLIENTE"            => $params["row"]["CDCLIENTE"],
                    "NRPROPOSFAS"          => $params["row"]["NRPROPOSFAS"],
                    "NRVERPROPFAS"         => $params["row"]["NRVERPROPFAS"],
                );
                $this->agendaServicosFactory->reverterSaldoHoras($dataParamsContrato);

                $nomeoperador = $this->agendaServicosFactory->getNomeOperador(array(
                    "CDOPERADOR" => $cdoperador,
                ));
                $dataParamsLogContratoServ = array(
                    "NMTABELA"      => "CONTRATOSERV",
                    "NMRESPOPER"    => $nomeoperador["data"][0]["NMOPERADOR"],
                    "CDORESPOPER"   => $cdoperador,
                    "IDTIPOOPER"    => "1",
                    "DSNMUSUOPER"   => "sm-tek",
                    "DSMETUSUOPER"  => "reverterSaldoHoras",
                    "DSOPERACAO"    => $params["row"]["DSOPERACAO"],
                    "CDPESQITEM"    => $params["row"]["NRPROPOSFAS"] . $params["row"]["NRVERPROPFAS"] . substr($params['row']["CDTIPOSERV"], strlen($params['row']['ROWNUM']) +1) . $params["row"]["NRSERVCONT"],
                );
                $this->agendaServicosFactory->insertLogOpera($dataParamsLogContratoServ);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function reverterSaldoHorasAction($params)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();
        try {
            foreach ($params["row"] as $row) {
                if ($row["USR_QTSERVCONT_BKP"] != null) {
                    $bkpContrato = array(
                        "QTSERVCONT"           => $row["USR_QTSERVCONT_BKP"],
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORG"                => $nrorg,
                        "CDTIPOSERV"           => substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1),
                        "CDCLIENTE"            => $row["CDCLIENTE"],
                        "NRPROPOSFAS"          => $row["NRPROPOSFAS"],
                        "NRVERPROPFAS"         => $row["NRVERPROPFAS"],
                    );
                    $this->agendaServicosFactory->reverterSaldoHorasBkp($bkpContrato);

                    $dataParamsContrato = array(
                        "USR_QTSERVCONT_BKP"   => null,
                        "USR_DTAGENDZERASALDO" => null,
                        "CDOPERULTATU"         => $cdoperador,
                        "NRORG"                => $nrorg,
                        "CDTIPOSERV"           => substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1),
                        "CDCLIENTE"            => $row["CDCLIENTE"],
                        "NRPROPOSFAS"          => $row["NRPROPOSFAS"],
                        "NRVERPROPFAS"         => $row["NRVERPROPFAS"],
                    );
                    $this->agendaServicosFactory->reverterSaldoHoras($dataParamsContrato);

                    $nomeoperador = $this->agendaServicosFactory->getNomeOperador(array(
                        "CDOPERADOR" => $cdoperador,
                    ));

                    $novoSaldo = $this->converterSegundosFormato($row["USR_QTSERVCONT_BKP"] * 3600);
                    $dsoperacao = "SALDO DE HORAS ALTERADO - Anterior: " . $row["SALDO"] . " / Novo: " . $novoSaldo;
                    $dataParamsLogContratoServ = array(
                        "NMTABELA"      => "CONTRATOSERV",
                        "NMRESPOPER"    => $nomeoperador["data"][0]["NMOPERADOR"],
                        "CDORESPOPER"   => $cdoperador,
                        "IDTIPOOPER"    => "1",
                        "DSNMUSUOPER"   => "sm-tek",
                        "DSMETUSUOPER"  => "reverterSaldoHoras",
                        "DSOPERACAO"    => $dsoperacao,
                        "CDPESQITEM"    => $row["NRPROPOSFAS"] . $row["NRVERPROPFAS"] . substr($row["CDTIPOSERV"], strlen($row['ROWNUM']) +1) . $row["NRSERVCONT"],
                    );
                    $this->agendaServicosFactory->insertLogOpera($dataParamsLogContratoServ);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function converterSegundosFormato($segundos)
    {
        $h = $segundos / 3600;
        $resto = $segundos % 3600;
        $m = $resto / 60;
        return $h . ":" . $m;
    }

    public function getLogAgendaServicos($params)
    {
        $dataParams = array(
            "CDPESQITEM"        => $params["CDPESQITEM"],
        );
        return $this->agendaServicosFactory->getLogAgendaServicos($dataParams);
    }
}
