-- ====================================================================
--                 Procedimento para posio de estoque
--                 ------------ ---- ------- -- -------
--
--  Parmetros: P_Filial     = Cdigo da filial a ser considerada.          (In)
--  ----------  P_Produto    = Produto a ser atualizado.                    (In)
--              P_Almoxarife = Cdigo do almoxarifado a ser considerado.    (In)
--              P_LocalEstoq = Cdigo do local de estoque a ser considerado.(In)
--              P_NRLOTEESTQ = Nmero do lote                               (In)
--              P_NRSUBLOTE  = Nmero do sub-lote                           (In)
--              P_QtdeEsto   = Quantidade em estoque.                       (Out)
--
-- @C:\Win_Prj\Sql\Oracle\0002_GERSTP_POSICAO_ESTOQUE_ATUAL.SQL
--
-- ====================================================================
CREATE OR REPLACE PROCEDURE POSICAO_ESTOQUE_ATUAL(
   P_FILIAL     IN POSESTDIA.CDFILIAL%TYPE,
   P_PRODUTO    IN POSESTDIA.CDPRODUTO%TYPE,
   P_ALMOXARIFE IN ALMOXARIFADO.CDALMOXARIFE%TYPE,
   P_LOCALESTOQ IN LOCALESTOQUE.CDLOCALESTOQ%TYPE,
   P_NRLOTEESTQ IN LOTE.NRLOTEESTQ%TYPE,
   P_NRSUBLOTE  IN LOTE.NRSUBLOTE%TYPE,
   P_QTDEESTO   OUT NUMBER)

AS
   V_QTDEESTO     NUMBER;
   V_QTDEANTE     NUMBER;

BEGIN

   SELECT NVL(ROUND(SUM(PO.QTTOTALESTOQ),5),0)
   INTO   V_QTDEESTO
   FROM POSESTATU PO, LOCALESTOQUE LO
   WHERE (PO.CDFILIAL      = P_FILIAL )
     AND (PO.CDPRODUTO     = P_PRODUTO)
     AND ((PO.CDALMOXARIFE = P_ALMOXARIFE) OR (P_ALMOXARIFE = 'T'))
     AND ((PO.CDLOCALESTOQ = P_LOCALESTOQ) OR (P_LOCALESTOQ = 'T'))
     AND ((PO.NRLOTEESTQ   = P_NRLOTEESTQ) OR (P_NRLOTEESTQ = 'T'))
     AND ((PO.NRSUBLOTE    = P_NRSUBLOTE ) OR (P_NRSUBLOTE  = 'T'))
     AND (PO.CDFILIAL      = LO.CDFILIAL )
     AND (PO.CDALMOXARIFE  = LO.CDALMOXARIFE)
     AND (PO.CDLOCALESTOQ  = LO.CDLOCALESTOQ)
     AND (LO.IDDISPCONPED  = 'S' OR LO.IDDISPCONPED IS NULL);

   P_QTDEESTO := V_QTDEESTO;

END;
/
