import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form,
    LoadingService,
    Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid, TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class Sm00006000ElementoController {
    private ModalFiltroElemento!: Modal;

    public ModalElementoAdd!: Modal;

    public ModalRelacServAdd!: Modal;

    public ModalCadastramentoAutomatico!: Modal;

    public visibleFrame: boolean = false;
    
    public visibleGrid: boolean = true;

    private CDSERVORCA: any = '';

    private clicks: any = 0;

    public currentRow: any = {};

    public currentRowRelacionamentoServico({ row }: any) {
        this.currentRow = row;
    }

    public async adicionarRelacionamentoServico() {
        this.ModalRelacServAdd = await ModalService.createFromJson('/sm_00006_000_elemento/Modal/sm_00006_000_relacionamento_servico_add', true);
        this.ModalRelacServAdd.show();
    }

    public closeModalRelacionamentoServicoAdd() {
        this.ModalRelacServAdd.destroy();
    }

    public async salvarCadastramentoAutomatico() {
        const form1 = Metadata.getInstance<Form>('modal-add-cadastramento-automatico-form');
        const grid = Metadata.getInstance<TekTreeGrid>('frame-servico-grid');

        if (Utilities.validateForm(form1)) {
            LoadingService.show();
            
            Http.post('/salvarCadastramentoAutomatico', {
                CDSERVORCA: Metadata.getInstance<TextInput>('modal-add-cadastramento-automatico-elemento-servico').value,
                CDSERVORCARELAC: Metadata.getInstance<TextInput>('modal-cadastramento-automatico-servico').value,
                QTMEDIDASERV: Metadata.getInstance<TextInput>('modal-cadastramento-automatico-quantidade-medida').value,
                CDTPMEDIDASERV: Metadata.getInstance<TextInput>('modal-cadastramento-automatico-tipo-medida').value,
                IDMEDIDAFIXA: Metadata.getInstance<TextInput>('modal-cadastramento-automatico-fixar').value,
            }).then(() => {
                // this.closeModalRelacionamentoServicoAdd();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async salvarRelacionamentoServicoEdit({ component }: any) {
        const grid = Metadata.getInstance<TekGrid>('frame-servico-grid');

        if (Utilities.validateGrid(grid)) {
            LoadingService.show();

            Http.post('/salvarRelacionamentoServicoEdit', {
                EDITEDROWS: component.getEditedRows(),
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async salvarRelacionamentoServicoAdd() {
        const form1 = Metadata.getInstance<Form>('modal-add-relacionamento-servico-form');
        const grid = Metadata.getInstance<TekTreeGrid>('frame-servico-grid');

        if (Utilities.validateForm(form1)) {
            LoadingService.show();

            Http.post('/salvarRelacionamentoServicoAdd', {
                CDSERVORCARELAC: Metadata.getInstance<TextInput>('modal-add-relacionamento-servico-servico').value,
                CDSERVORCA: this.CDSERVORCA,
                QTMEDIDASERV: Metadata.getInstance<TextInput>('modal-add-relacionamento-servico-quantidade-medida').value,
                CDTPMEDIDASERV: Metadata.getInstance<TextInput>('modal-add-relacionamento-servico-tipo-medida').value,
                IDMEDIDAFIXA: Metadata.getInstance<TextInput>('modal-add-relacionamento-servico-fixar').value,
            }).then(() => {
                this.closeModalRelacionamentoServicoAdd();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    get tituloDetail() {
        return this.currentRow.NMSERVORCA;
    }

    public dialogDeletarRelacionamentoServico({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Sm00006000ElementoController.aposConfirmarDeleteRelacionamentoServico');
    }

    public async aposConfirmarDeleteRelacionamentoServico() {
        const grid = Metadata.getInstance<TekTreeGrid>('frame-servico-grid');
        LoadingService.show();

        Http.post('/deletarRelacionamentoServico', {
            NRS: grid.selectedRows.map((element: any) => element.NRRELACSERVICO),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public excluirElementos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Sm00006000ElementoController.aposConfirmarDelete');
    }

    public async aposConfirmarDelete() {
        const grid = Metadata.getInstance<TekTreeGrid>('grid-elemento-principal');
        LoadingService.show();

        Http.post('/excluirElementos', {
            NRS: grid.selectedRows.map((element: any) => element.CDSERVORCA),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public setFilterElementoSuperior({ component }: any) {
        component.datasource.setFilter({
            IDATIVO: 'S',
            IDTIPOELEMENTO: 'G',
        });
    }

    public changeFormacaoPreco({ component }: any) {
        const medidaVariavel = Metadata.getInstance<Select>('frame-add-elemento-medida-variavel');
        const identificacao = Metadata.getInstance<TextInput>('frame-add-elemento-identificacao');
        const tipoCalculo = Metadata.getInstance<TextInput>('calculo-padrao-modal-add');

        const ordenacao = Metadata.getInstance<TextInput>('frame-add-elemento-ordenacao');

        if (component.value === 'S') {
            medidaVariavel.readonly = true;
            identificacao.readonly = false;
            tipoCalculo.readonly = false;
            ordenacao.addValidation('required', {});
            identificacao.addValidation('required', {});
        } else if (component.value === 'N') {
            medidaVariavel.readonly = false;
            identificacao.readonly = true;
            tipoCalculo.readonly = true;

            ordenacao.removeValidation('required');
            identificacao.removeValidation('required');
        }
    }

    public changeTipoCalculoPadrao({ component }: any) {
        const calculoPadrao = Metadata.getInstance<Select>('calculo-padrao-modal-add');
        const expressaoPadrao = Metadata.getInstance<TextInput>('expressao-padrao-modal-add');
        const percentualPadrao = Metadata.getInstance<TextInput>('percentual-padrao-modal-add');

        if (component.value === 'E') {
            calculoPadrao.readonly = false;
            percentualPadrao.readonly = false;
            expressaoPadrao.readonly = false;
            expressaoPadrao.addValidation('required', {});
        } else {
            calculoPadrao.readonly = false;
            percentualPadrao.readonly = true;
            expressaoPadrao.readonly = true;
            percentualPadrao.value = null;
            expressaoPadrao.value = null;
            expressaoPadrao.removeValidation('required');
        }
    }

    public changeFormacaoPrecoEdit({ component }: any) {
        const medidaVariavel = Metadata.getInstance<Select>('frame-elemento-medida-variavel');
        const identificacao = Metadata.getInstance<TextInput>('frame-elemento-identificacao');
        const tipoCalculo = Metadata.getInstance<TextInput>('frame-elemento-calculo-padrao');

        const ordenacao = Metadata.getInstance<TextInput>('frame-elemento-ordenacao');

        if (component.value === 'S') {
            medidaVariavel.readonly = true;
            identificacao.readonly = false;
            tipoCalculo.readonly = false;
            ordenacao.addValidation('required', {});
            identificacao.addValidation('required', {});
        } else if (component.value === 'N') {
            medidaVariavel.readonly = false;
            identificacao.readonly = true;
            tipoCalculo.readonly = true;

            ordenacao.removeValidation('required');
            identificacao.removeValidation('required');
        }
    }

    public changeTipoCalculoPadraoEdit({ component }: any) {
        const calculoPadrao = Metadata.getInstance<Select>('frame-elemento-calculo-padrao');
        const expressaoPadrao = Metadata.getInstance<TextInput>('frame-elemento-expressao-padrao');
        const percentualPadrao = Metadata.getInstance<TextInput>('frame-elemento-percentual-padrao');

        if (component.value === 'E') {
            calculoPadrao.readonly = false;
            percentualPadrao.readonly = false;
            expressaoPadrao.readonly = false;
            expressaoPadrao.addValidation('required', {});
        } else {
            calculoPadrao.readonly = false;
            percentualPadrao.readonly = true;
            expressaoPadrao.readonly = true;
            percentualPadrao.value = null;
            expressaoPadrao.value = null;
            expressaoPadrao.removeValidation('required');
        }
    }

    public async addNewRow() {
        this.ModalElementoAdd = await ModalService.createFromJson('/sm_00006_000_elemento/Modal/sm_00006_000_elemento_add', true);
        this.ModalElementoAdd.show();
    }

    public closeModalElementoAdd() {
        this.ModalElementoAdd.destroy();
    }

    public rowClickFunction({ row }: any) {
        this.clicks += 1;

        setTimeout(() => {
            if (this.clicks === 2) {
                this.openPreview({ row });
            }
            this.clicks = 0;
        }, 1000);
    }

    public async salvarElementoAdd() {
        const form1 = Metadata.getInstance<Form>('frame-add-elemento-form');
        const form2 = Metadata.getInstance<Form>('form-infoAdicional-modal-add');
        const grid = Metadata.getInstance<TekTreeGrid>('grid-elemento-principal');

        if (Utilities.validateForm(form1) && Utilities.validateForm(form2)) {
            LoadingService.show();

            Http.post('/salvarElementoAdd', {
                IDATIVO: Metadata.getInstance<TextInput>('frame-add-elemento-ativo').value,
                NMSERVORCA: Metadata.getInstance<TextInput>('frame-add-elemento-nome').value,
                IDELEMENTOPAI: Metadata.getInstance<TextInput>('frame-add-elemento-superior').value,
                CDTPMEDIDAVARIAVEL: Metadata.getInstance<TextInput>('frame-add-elemento-medida-variavel').value,
                IDORDENAELEMENTO: Metadata.getInstance<TextInput>('frame-add-elemento-ordenacao').value,
                IDFORMACAOPRECO: Metadata.getInstance<TextInput>('frame-add-elemento-formacao-preco').value,
                IDALTERAFORMPREC: Metadata.getInstance<TextInput>('frame-add-elemento-permite-alteracao').value,
                DSSERVORCA: Metadata.getInstance<TextInput>('frame-add-elemento-descricao').value,
                CDCENTCUST: Metadata.getInstance<TextInput>('frame-add-elemento-centro-custo').value,
                IDITEMCALCULO: Metadata.getInstance<TextInput>('frame-add-elemento-identificacao').value,
                IDTIPOCALCULO: Metadata.getInstance<TextInput>('calculo-padrao-modal-add').value,
                DSEXPRESSAOCALC: Metadata.getInstance<TextInput>('expressao-padrao-modal-add').value,
                VRPERCCALCULO: Metadata.getInstance<TextInput>('percentual-padrao-modal-add').value,
            }).then(() => {
                this.closeModalElementoAdd();
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async salvarElementoEdit() {
        const form1 = Metadata.getInstance<Form>('frame-elemento-form');
        const form2 = Metadata.getInstance<Form>('form-informacoesAdicionaisFrameElemento');
        const grid = Metadata.getInstance<TekTreeGrid>('grid-elemento-principal');

        if (Utilities.validateForm(form1) && Utilities.validateForm(form2)) {
            LoadingService.show();

            Http.post('/salvarElementoEdit', {
                CDSERVORCA: this.CDSERVORCA,
                IDATIVO: Metadata.getInstance<TextInput>('frame-elemento-ativo').value,
                NMSERVORCA: Metadata.getInstance<TextInput>('frame-elemento-nome').value,
                IDELEMENTOPAI: Metadata.getInstance<TextInput>('frame-elemento-superior').value,
                CDTPMEDIDAVARIAVEL: Metadata.getInstance<TextInput>('frame-elemento-medida-variavel').value,
                IDORDENAELEMENTO: Metadata.getInstance<TextInput>('frame-elemento-ordenacao').value,
                IDFORMACAOPRECO: Metadata.getInstance<TextInput>('frame-elemento-formacao-preco').value,
                IDALTERAFORMPREC: Metadata.getInstance<TextInput>('frame-elemento-permite-alteracao').value,
                DSSERVORCA: Metadata.getInstance<TextInput>('frame-elemento-descricao').value,
                CDCENTCUST: Metadata.getInstance<TextInput>('frame-elemento-centro-custo').value,
                IDITEMCALCULO: Metadata.getInstance<TextInput>('frame-elemento-identificacao').value,
                IDTIPOCALCULO: Metadata.getInstance<TextInput>('frame-elemento-calculo-padrao').value,
                DSEXPRESSAOCALC: Metadata.getInstance<TextInput>('frame-elemento-expressao-padrao').value,
                VRPERCCALCULO: Metadata.getInstance<TextInput>('frame-elemento-percentual-padrao').value,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public openPreview({ row }: any) {
        this.currentRow = row;
        this.CDSERVORCA = row.CDSERVORCA;

        Metadata.getInstance<Select>('frame-elemento-ativo').value = row.IDATIVO;
        Metadata.getInstance<TextInput>('frame-elemento-nome').value = row.NMSERVORCA;
        Metadata.getInstance<TextInput>('frame-elemento-superior').value = row.IDELEMENTOPAI;
        Metadata.getInstance<Select>('frame-elemento-medida-variavel').value = row.CDTPMEDIDAVARIAVEL;
        Metadata.getInstance<TextInput>('frame-elemento-ordenacao').value = row.IDORDENAELEMENTO;
        Metadata.getInstance<Select>('frame-elemento-formacao-preco').value = row.IDFORMACAOPRECO;
        Metadata.getInstance<TextInput>('frame-elemento-identificacao').value = row.IDITEMCALCULO;
        Metadata.getInstance<Select>('frame-elemento-permite-alteracao').value = row.IDALTERAFORMPREC;
        Metadata.getInstance<TextInput>('frame-elemento-descricao').value = row.DSSERVORCA;
        Metadata.getInstance<TextInput>('frame-elemento-centro-custo').value = row.CDCENTCUST;
        Metadata.getInstance<TextInput>('frame-elemento-cdservorca').value = this.CDSERVORCA;

        this.visibleGrid = false;
        this.visibleFrame = true;
        this.filtrarRelacionamentoServico();
    }

    public filtrarRelacionamentoServico() {
        try {
            const grid = Metadata.getInstance<TekGrid>('frame-servico-grid');
            this.setFilterGrid(grid, this.CDSERVORCA);
        } catch (e) {
            // console.error(e);
        }
    }

    public setFilterGrid(grid: TekGrid, NR: any) {
        const codigo = NR || -1;
        return grid.datasource.setFilter({
            NR: codigo,
        });
    }

    public backToPageMain() {
        this.visibleGrid = true;
        this.visibleFrame = false;
    }

    public async createModalFiltroElemento() {
        this.ModalFiltroElemento = await ModalService.createFromJson('/sm_00006_000_elemento/Modal/sm_00006_000_elemento_filtro', true);
        this.openModalFiltroElemento();
    }

    public destroyModalFiltroElemento() {
        this.ModalFiltroElemento.destroy();
    }

    public openModalFiltroElemento() {
        this.ModalFiltroElemento.show();
    }

    public closeModalFiltroElemento() {
        this.ModalFiltroElemento.hide();
    }

    public eventFilter({ event }: any) {
        this.openModalFiltroElemento();
        event.preventDefault();
    }

    public async createModalCadastramentoAutomatico() {
        this.ModalCadastramentoAutomatico = await ModalService.createFromJson('/sm_00006_000_elemento/Modal/sm_00006_000_cadastramento_automatico', true);
        this.openModalCadastramentoAutomatico();
    }

    public destroyModalCadastramentoAutomatico() {
        this.ModalCadastramentoAutomatico.destroy();
    }

    public openModalCadastramentoAutomatico() {
        this.ModalCadastramentoAutomatico.show();
    }

    public closeModalCadastramentoAutomatico() {
        this.ModalCadastramentoAutomatico.hide();
    }

    public filtrarElemento() {
        const grid = Metadata.getInstance<TekTreeGrid>('grid-elemento-principal');
        const form = Metadata.getInstance<Form>('form-filter');

        if (Utilities.validateForm(form)) {
            grid.datasource.setFilter({
                IDATIVO: Metadata.getInstance<Select>('filtro-idativo').value,
                NOME: Metadata.getInstance<TextInput>('filtro-nome').value,
                CDTPMEDIDAVARIAVEL: Metadata.getInstance<SelectMultiple>('filtro-medida-variavel').value,
                RELACSERVICO: Metadata.getInstance<SelectMultiple>('filtro-relacionamento-servico').value,
                FORMACAOPRECO: Metadata.getInstance<SelectMultiple>('filtro-formacao-preco').value,
                CDCENTCUST: Metadata.getInstance<SelectMultiple>('filtro-centro-custo').value,
            });
            this.ModalFiltroElemento.hide();
        }
    }

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }
}
